( cd usr/lib ; rm -rf libesd.so )
( cd usr/lib ; ln -sf libesd.so.0.2.32 libesd.so )
( cd usr/lib ; rm -rf libesd.so.0 )
( cd usr/lib ; ln -sf libesd.so.0.2.32 libesd.so.0 )
( cd usr/lib ; rm -rf libesddsp.so )
( cd usr/lib ; ln -sf libesddsp.so.0.2.32 libesddsp.so )
( cd usr/lib ; rm -rf libesddsp.so.0 )
( cd usr/lib ; ln -sf libesddsp.so.0.2.32 libesddsp.so.0 )
